package Apache::CrowdAuth;

use 5.008000;
use strict;
use warnings;

require Exporter;

our @ISA = qw(Exporter);

our $VERSION = '0.03';


# Use correct API for loaded version of mod_perl.
#
BEGIN {

    unless ( $INC{'mod_perl.pm'} ) {

        my $class = 'mod_perl';

        if ( exists $ENV{MOD_PERL_API_VERSION} && $ENV{MOD_PERL_API_VERSION} == 2 ) {
            $class = 'mod_perl2';
        }

        eval "require $class";
    }

    my @import = qw( OK HTTP_UNAUTHORIZED SERVER_ERROR );

    if ( $mod_perl::VERSION >= 1.999022 ) { # mod_perl 2.0.0 RC5
        require Apache2::RequestRec;
        require Apache2::RequestUtil;
        require Apache2::RequestIO;
        require Apache2::Log;
        require Apache2::Connection;
        require Apache2::Const;
        require Apache2::Access;
        Apache2::Const->import(@import);
     }
     elsif ( $mod_perl::VERSION >= 1.99 ) {
        require Apache::RequestRec;
        require Apache::RequestUtil;
        require Apache::RequestIO;
        require Apache::Log;
        require Apache::Connection;
        require Apache::Const;
        require Apache::Access;
        Apache::Const->import(@import);
    }
    else {
        require Apache;
        require Apache::Log;
        require Apache::Constants;
        Apache::Constants->import(@import);
    }
}

use constant MP2 => $mod_perl::VERSION >= 1.999022 ? 1 : 0;

# Uncomment the following line (and comment out the line below it) to
# enable debug output of the SOAP messages.
# use SOAP::Lite +trace => qw (debug);
use SOAP::Lite;

# Global to hold crowd SOAP service url. Can be overriden with the 
# CrowdSOAPURL apache config parameter.
my $HOST = "http://localhost:8080/crowd/services/SecurityServer";

# Namespace
my $NS = "urn:SecurityServer";

# namespace attribute used in SOAP call creation.
my $XMLNS = 'http://authentication.integration.crowd.atlassian.com';

# Entry Point
#
sub handler {
	my $r = shift;

	my $rlog = $r->log;
	
	my ($status, $password) = $r->get_basic_auth_pw;
	return $status unless $status == OK;
	
	my $user = $r->user;
	unless($user and $password) {
		 $r->note_basic_auth_failure;
		 $rlog->info("Both a username and password must be provided");
		 return HTTP_UNAUTHORIZED;
	}
	 
	# Get parameters from the apache conf file
	my $app_name = $r->dir_config('CrowdAppName');
	my $app_credential = $r->dir_config('CrowdAppPassword');
	
	# Both the application name and credential password need to be defined.
	if(!defined($app_name) || !defined($app_credential)) {
		$r->log_error("CrowdAuth: CrowdAppName or CrowdAppPassword is not defined");
		$r->note_basic_auth_failure;
		return HTTP_UNAUTHORIZED;
	}
	
	my $soaphost = $r->dir_config('CrowdSOAPURL');
	if(defined($soaphost)) {
		$HOST = $soaphost;
	}
	
	$rlog->debug('CrowdAuth: About to auth app ['.$app_name.','.$app_credential.']...');
		
	my $apptoken;
	if($apptoken = authenticate_app($r, $app_name, $app_credential)) {
		$rlog->debug('CrowdAuth: auth app OK...'.$apptoken);
		# We've got a new app token - try to authenticate the user
		if(!authenticate_principal($r, $app_name, $apptoken, $user, $password)) {
			# failed to auth user.
			$rlog->warn('CrowdAuth: Failed to authenticate '.$user.'.');
			$r->note_basic_auth_failure;
			return HTTP_UNAUTHORIZED;
		}
	} else {
		$r->log_error('CrowdAuth: Failed to authenticate application.');
		# failed to auth app.
		$r->note_basic_auth_failure;
		return HTTP_UNAUTHORIZED;
	}
		
	$rlog->info('CrowdAuth: Principal '.$user.' authenticated OK');
	return OK
}

sub make_soap_call {
	my ($method, @params) = @_;
	
	my $search = SOAP::Lite
		->readable(1)
		->xmlschema('http://www.w3.org/2001/XMLSchema')
		->on_action( sub { return '""';} )
		->proxy($HOST)
		->uri($NS)
		->default_ns($XMLNS);
	
	my $app_method = SOAP::Data->name($method)
		->uri($NS);
	
	my $som = $search->call($app_method => @params);
	
	return $som;
}

# Authenticate the application
sub authenticate_app {
	my ($r, $app_name, $app_credential) = @_;
	my $rlog = $r->log;
	
	if(!defined($app_name) || !defined($app_credential)) {
		return 0;
	}
	
	my @app_params = (
	SOAP::Data->name('in0' =>
		\SOAP::Data->value(
			SOAP::Data->name('credential' => \SOAP::Data->value(
				SOAP::Data->name('credential' => $app_credential)))->attr({xmlns => $XMLNS}),
				SOAP::Data->name('name' => $app_name)->attr({xmlns => $XMLNS}),
				SOAP::Data->name('validationFactors' => undef)->attr({xmlns => $XMLNS})
	)));
	
	$rlog->debug("CrowdAuth: Logging in Application " . $app_name . " with password ". $app_credential . "...");
	
	my $app_som = make_soap_call('authenticateApplication', @app_params);
	
	if ($app_som->fault) {
		$r->log_error("CrowdAuth: Failed to authenticate application: " . $app_som->faultcode . ":" . $app_som->faultstring);
		return 0;
	}
	
	# get the token
	my $appToken = $app_som->valueof('//token');
	
	if(!defined($appToken)) {
		return 0;
	}
		
	$r->log->debug("CrowdAuth: Application authenticated, token = " . $appToken);
	return $appToken;
}

# authenticate the principal
sub authenticate_principal {
	
	my ($r, $app_name, $appToken, $principal_name, $principal_credential) = @_;
	my $rlog = $r->log;
	
	if(!defined($appToken)) {
		$rlog->debug("CrowdAuth: No App Token supplied [" . $principal_name . "]...");
		return 0;
	}

	my $principal_method = SOAP::Data->name('authenticatePrincipal')
	->uri($NS);
	
	my @principal_params = (
	SOAP::Data->name('in0' =>
		\SOAP::Data->value(
				SOAP::Data->name('name' => $app_name)->attr({xmlns => $XMLNS}),
				SOAP::Data->name('token' => $appToken)->attr({xmlns => $XMLNS}),
	)),
	SOAP::Data->name('in1' =>
	\SOAP::Data->value(
		SOAP::Data->name('application' => $app_name)->attr({xmlns => $XMLNS}),
		SOAP::Data->name('credential' => \SOAP::Data->value(
			SOAP::Data->name('credential' => $principal_credential)))->attr({xmlns => $XMLNS}),
			SOAP::Data->name('name' => $principal_name)->attr({xmlns => $XMLNS}),
			SOAP::Data->name('validationFactors' => undef)->attr({xmlns => $XMLNS})
			))
	);
	
	$rlog->debug("CrowdAuth: Logging in Principal [" . $principal_name . ",". $principal_credential . ",".$appToken."]...");
	my $principal_som = make_soap_call('authenticatePrincipal', @principal_params);
	
	if ($principal_som->fault) { # will be defined if Fault element is in the message
		$rlog->warn("CrowdAuth: Failed to authenticate principal : " . $principal_som->faultcode . ":" . $principal_som->faultstring); 
		return 0;
	} else {
		my $principalToken = $principal_som->valueof('//authenticatePrincipalResponse/out');

		if(defined($principalToken)) {
			$rlog->debug("CrowdAuth: ".$principal_name. " authenticated, token = " . $principalToken);
			return 1;
		} else {
			$r->log_error("CrowdAuth: User token not found in SOAP response for user ".$principal_name);
			return 0;
		}
	}
	return 0;
}



1;
__END__


=head1 NAME

Apache::CrowdAuth - Apache authentication handler that uses Atlassian Crowd.

=head1 SYNOPSIS

<Location /location>
  AuthName crowd
  AuthType Basic

  PerlAuthenHandler Apache::CrowdAuth
  PerlSetVar CrowdAppName appname
  PerlSetVar CrowdAppPassword apppassword
  PerlSetVar CrowdSOAPURL http://localhost:8080/crowd/services/SecurityServer

  require valid-user
</Location>

=head1 DESCRIPTION

This Module allows you to configure Apache to use Atlassian Crowd to 
handle basic authentication.
	
See http://confluence.atlassian.com/display/CROWD/Integrating+Crowd+with+Apache

for full documentation.

=head2 EXPORT

None by default.



=head1 SEE ALSO

http://www.atlassian.com/crowd

=head1 AUTHOR

Atlassian.

=head1 COPYRIGHT AND LICENSE

Copyright (C) 2007 by Atlassian

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.8.5 or,
at your option, any later version of Perl 5 you may have available.

=cut
